from __future__ import (print_function, unicode_literals, division,
    absolute_import)

import atexit
from logging import getLogger
import shutil

from six.moves.urllib.parse import urlparse, urlunparse


log = getLogger('binstar.git')

def is_url(path):
    url = urlparse(path)
    if url.scheme:
        return True

def get_urlpath(path):
    url = urlparse(path)
    return url.path[1:]  # path is /a/b (want to remove first slash)



def clone_repo(path):
    url = urlparse(path)
    fragment = url.fragment
    scheme = url.scheme[4:]

    git_url = urlunparse((scheme, url.netloc, url.path, '', '', ''))
    import tempfile
    from subprocess import check_call
    tmp_dir = tempfile.mkdtemp('.git', 'binstar-build')
    log.info(' '.join(['git', 'clone', git_url, tmp_dir]))
    check_call(['git', 'clone', git_url, tmp_dir])
    log.info(' '.join(['git', 'checkout', fragment]))
    check_call(['git', 'checkout', fragment], cwd=tmp_dir)

    def rmrepo():
        log.info("Removing git temp git repo")
        shutil.rmtree(tmp_dir, ignore_errors=True)

    atexit.register(rmrepo)

    return tmp_dir



'''
Save build info
'''

from __future__ import (print_function, unicode_literals, division,
    absolute_import)

from argparse import RawDescriptionHelpFormatter
import logging
import re

from binstar_build_client import BinstarBuildAPI
from binstar_client import errors
from binstar_client.utils import get_binstar, PackageSpec
from binstar_client.utils import package_specs
from six.moves.urllib.parse import urlparse

log = logging.getLogger('binstar.build')

def main(args):

    binstar = get_binstar(args, cls=BinstarBuildAPI)

    # Force user auth

    package_name = None

    url = urlparse(args.url)


    if not args.package:
        package_name = url.split('/')[1]
        log.info("Using repo name '%s' as the pkg name." % package_name)
        user = binstar.user()
        user_name = user['login']
        args.package = PackageSpec(user_name, package_name)

    binstar = get_binstar(args, cls=BinstarBuildAPI)

    try:
        _ = binstar.package(args.package.user, args.package.name)
    except errors.NotFound:
        raise errors.UserError("Package %s does not exist" % (args.package,))

    log.info("Submitting the following repo for package creation: %s" % args.url)

    if url.netloc != 'github.com':
        raise errors.UserError("Currently only github.com urls are supported (got %s)" % url.netloc)

    pat = re.compile('^/(?P<repo>[\w-]+/[\w-]+)$')
    match = pat.match(url.path)
    if not match:
        raise errors.UserError("URL path '%s' is not a git repo" % url.path)

    groups = match.groupdict()
    repo = groups.get('repo')
    ghowner, ghrepo = repo.split('/', 1)

    binstar.add_ci(args.package.user, args.package.name,
                   ghowner=ghowner, ghrepo=ghrepo,
                   channels=args.channels, queue=args.queue, sub_dir=args.sub_dir,
                   email=args.email)

    log.info("CI Added to package %s", args.package)

def add_parser(subparsers):
    parser = subparsers.add_parser('save',
                                      help='Save build info',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter,
                                      )

    parser.add_argument('url',
                        help='The http github url to the repo')

    parser.add_argument('-p', '--package',
                       help="The binstar package namespace to upload the build to",
                       metavar='USER/PACKAGE',
                       type=package_specs)

    parser.add_argument('--sub-dir',
                       help="The sub directory within the git repository (github url submits only)")

    parser.add_argument('--channel', action='append', dest='channels',
                       help="Upload targets to this channel")

    parser.add_argument('--queue',
                       help="Build on this queue")

#     parser.add_argument('--buildhost',
#                         help="The host name of the intended build worker")
#
#     parser.add_argument('--dist',
#                         help=("The os distribution of intended build worker (e.g centos, ubuntu) "
#                               "Use 'binstar-build queue' to view the workers")
#                         )
#
#     parser.add_argument('--platform',
#                         help=("The platform to run (e.g linux-64, win-64, osx-64, etc) "
#                               "(default: all the platforms in the .binstar.yaml file)")
#                         )
    parser.add_argument('--email', action='append',
                        help="Binstar usernames or email adresses to email when the build completes"
                        )

    parser.set_defaults(main=main)




'''
Build Queue
'''
from __future__ import (print_function, unicode_literals, division,
    absolute_import)
from binstar_client.utils import get_binstar, bool_input
from binstar_build_client import BinstarBuildAPI
from argparse import RawDescriptionHelpFormatter
from dateutil.parser import parse as parse_date
from binstar_client.commands.authorizations import format_timedelta
from binstar_client import errors

def show_queue(queue):
    queue_owner = queue.get('owner')
    platforms = ', '.join(queue.get('platforms', []))

    if queue_owner:
        queue_name = '{0}/{1}'.format(queue_owner['login'], queue['_id'])
    else:
        queue_name = queue['_id']
    print('{queue_name:30} [{platforms}]'.format(**locals()))
    for worker in queue.get('workers', []):
        print(' + Worker hostname:{worker[hostname]:15} platform:{worker[platform]:15} dist:{worker[dist]:15}'.format(**locals()))

        try:
            last_seen = parse_date(worker['last_seen'])
            last_seen = '%s ago' % format_timedelta(last_seen, False)
        except TypeError:
            last_seen = worker['last_seen']

        print('   - Id {0[id]}'.format(worker))
        print('   - Last seen {0}'.format(last_seen))
        print('   - binstar-build v{0[binstar_build_version]} (binstar v{0[binstar_version]})'.format(worker))
    if not queue.get('workers', []):
        print(" + No build workers attached to this queue")
    print()

def show_queues(bs, username):
    print()
    for queue in bs.build_queues(username):
        show_queue(queue)

def main(args):
    bs = get_binstar(args, cls=BinstarBuildAPI)

    if args.organization:
        username = args.organization
    else:
        user = bs.user()
        username = user['login']

    if args.create:

        if not args.platform:
            raise errors.BinstarError("Must specify at least one platform")

        bs.add_build_queue(username, args.queue, args.platform)
        print("Created queue %s" % args.queue)
        return

    if args.queue:
        queue = bs.build_queue(username, args.queue)

    if args.remove:
        if queue.get('workers'):
            prompt = ('This build queue still has workers attached. '
                      'Are you sure you want to remove it')
            if not bool_input(prompt, False):
                print("Not removing queue")
                return
        bs.remove_build_queue(username, args.queue)
        print("Removed queue %s" % args.queue)
        return

    if args.remove_worker:
        bs.remove_worker(username, args.queue, args.remove_worker)
        print("Removed worker %s from queue %s" % (args.remove_worker, args.queue))
        return


    if args.queue:
        print()
        show_queue(queue)
    else:
        show_queues(bs, username)

def add_parser(subparsers):
    parser = subparsers.add_parser('queue',
                                      help='Inspect build queue',
                                      description=__doc__,
                                      formatter_class=RawDescriptionHelpFormatter,
                                      )

    parser.add_argument('-o', '--organization',
                        help='Act as an organization')
    parser.add_argument('-q', '--queue', metavar='Q',
                        help='Specify a queue to perform an operation on')
    parser.add_argument('-r', '--remove', action='store_true',
                        help='Remove the queue specified with the -q/--queue option')
    parser.add_argument('-c', '--create', action='store_true',
                        help='Create a new queue')
    parser.add_argument('-p', '--platform', action='append', default=[],
                        help='Append a platform to the list (use with -c/--create option)')
    parser.add_argument('--remove-worker', metavar='WORKER_ID',
                        help='Remove a worker from a queue')

    parser.set_defaults(main=main)
